/**
 * 
 */
package org.evolvis.xana.swt;

import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.swing.Action;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Decorations;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.evolvis.xana.action.ActionContainer;
import org.evolvis.xana.action.ActionRegistry;
import org.evolvis.xana.action.TarentGUIAction;

/**
 * @author Fabian K&ouml;ster (f.koester@tarent.de) tarent GmbH Bonn
 *
 */
public class SWTMenuBar implements ActionContainer {

	protected String uniqueName;
	protected ResourceBundle resourceBundle;
	protected Menu menu;
	protected Decorations parent;
	protected static final String DEFAULT_MENU = "Extras";
	protected final static Logger logger = Logger.getLogger(SWTMenuBar.class.getName());
	protected Map<String, Menu> menus = new HashMap<String, Menu>();
	
	public SWTMenuBar( String uniqueName, ResourceBundle resourceBundle, Decorations parent) {
		menu = new Menu(parent, SWT.BAR);
        this.uniqueName = uniqueName;
        this.resourceBundle = resourceBundle;
        this.parent = parent;
    }
	
	/**
	 * @see org.evolvis.xana.action.ActionContainer#attachGUIAction(javax.swing.Action, java.lang.String)
	 */
	public void attachGUIAction(final Action action, String menuPath) {
		
		Menu submenu = getSubMenu(getMenuSteps(removeMainMenuPriority(menuPath)));
		
		MenuItem item = new MenuItem (submenu, SWT.PUSH);
		item.addListener (SWT.Selection, new Listener () {
			public void handleEvent (Event e) {
				action.actionPerformed(new ActionEvent(menu, 0, ""));
			}
		});
		item.setText(action.getValue(Action.NAME).toString());
	}

	/**
	 * @see org.evolvis.xana.action.ActionContainer#getContainerUniqueName()
	 */
	public String getContainerUniqueName() {
		return uniqueName;
	}

	/**
	 * @see org.evolvis.xana.action.ActionContainer#initActions()
	 */
	public void initActions() {
		Iterator<TarentGUIAction> mbIt = ActionRegistry.getInstance().getActions(getContainerUniqueName()).iterator();
		
		while(mbIt.hasNext()) {
			TarentGUIAction action = mbIt.next();
			attachGUIAction(action, action.getMenuPath(getContainerUniqueName()));
		}
	}

	/**
	 * @see org.evolvis.xana.action.ActionContainer#removeGUIAction(javax.swing.Action)
	 */
	public void removeGUIAction(Action arg0) {
		// TODO Auto-generated method stub

	}
	
	public Menu getMenu() {
		return menu;
	}
	
    protected String[] getMenuSteps( String menuPath ) {
        String[] menuSteps;
        if ( null == menuPath || "".equals( menuPath ) ) {
            menuSteps = new String[] { DEFAULT_MENU };
        }
        else {
            String[] menuPathParts = menuPath.split( ":" );
            menuSteps = menuPathParts[0].split( "/" );
        }
        return menuSteps;
    }
    
    protected String removeMainMenuPriority(String menuPath)
    {
    	String[] menuPathParts = null;
    	if(menuPath != null)
    		menuPathParts = menuPath.split(":");
    	else
    		return null;
    	
    	// check if the menuPath starts with a digit
    	if(menuPathParts != null && menuPathParts.length > 0 && Character.isDigit(menuPathParts[0].charAt(0)))
    	{
    		// this menuPath contains a priority-value for the main-menu. remove it
    		return menuPath.substring(menuPath.indexOf(':')+1);
    	}
    	
    	// does not contain a priority-value for the main-menu. return without change
    	return menuPath;
    }
    
    protected Menu getSubMenu(String[] menuPath) {
    	Menu submenu = menus.get(menuPath[0]);
    	
    	if(submenu == null) {
    		MenuItem menuBarItem = new MenuItem (menu, SWT.CASCADE);
    		menuBarItem.setText (menuPath[0]);
    		submenu = new Menu (parent, SWT.DROP_DOWN);
    		menuBarItem.setMenu (submenu);
    		menus.put(menuPath[0], submenu);
    	}
    	
    	return submenu;
    }
}